async function renderPopup() {
  document.querySelector('#GPTSummarizer__root')?.remove();

  const $root = document.createElement('div');
  $root.id = 'GPTSummarizer__root';
  $root.attachShadow({ mode: 'open' });

  const { POPUP_TEMPLATE } = await import(
    chrome.runtime.getURL('js/config.js')
  );

  $root.shadowRoot.innerHTML = `<link href="${chrome.runtime.getURL(
    'lib/bootstrap.css',
  )}" rel="stylesheet" type="text/css" />
  ${POPUP_TEMPLATE}`;

  const $settings = $root.shadowRoot.querySelector('#GPTSummarizer__settings');
  $settings.addEventListener('click', () => {
    chrome.runtime.sendMessage('OPEN_OPTIONS_PAGE');
  });

  const $close = $root.shadowRoot.querySelector('#GPTSummarizer__close');
  $close.addEventListener('click', () => $root.remove());

  document.body.parentElement.appendChild($root);
  setTimeout(() => {
    $root.shadowRoot.querySelector('#GPTSummarizer__popup').style.opacity = 1;
  }, 100);
  return $root.shadowRoot;
}

(async () => {
  const $root = await renderPopup();
  chrome.runtime.sendMessage('GET_SETTINGS', async (settings) => {
    const $output = $root.querySelector('#GPTSummarizer__output');
    try {
      const { getArticle, chat } = await import(
        chrome.runtime.getURL('js/utils.js')
      );
      const article = await getArticle();

      await chat(
        [
          {
            role: 'system',
            content: `You are GPT-4. ${settings.prompt}`,
          },
          {
            role: 'user',
            content: `${settings.prompt}\narticle:"""${article}"""`,
          },
        ],
        settings,
        (data) => {
          $output.innerHTML += data;
        },
      );

      await import(chrome.runtime.getURL('lib/pangu.min.js'));
      $output.innerHTML = window.pangu.spacing($output.innerHTML);
    } catch (error) {
      $output.innerHTML = error;
    }

    const $copy = $root.querySelector('#GPTSummarizer__copy');
    $copy.addEventListener('click', () => {
      const textarea = document.createElement('textarea');
      textarea.value = `${$output.innerHTML}\n${window.location.href}`;
      document.body.appendChild(textarea);
      textarea.select();
      document.execCommand('copy');
      document.body.removeChild(textarea);

      $copy.classList.add('copied');
      setTimeout(() => $copy.classList.remove('copied'), 2000);
    });

    $copy.classList.toggle('d-none');
    $root.querySelector('#GPTSummarizer__loading').classList.toggle('d-none');
    $root.querySelector('#GPTSummarizer__title').classList.toggle('d-none');
  });
})();
