import { DEFAULT_SETTINGS } from './config.js';
import { getLocalStorage } from './utils.js';

(() => {
  async function getSettings() {
    const result = (await getLocalStorage('settings')) || {};
    return { ...DEFAULT_SETTINGS, ...result };
  }

  chrome.runtime.onInstalled.addListener(({ reason }) => {
    if (reason === 'install') {
      chrome.runtime.openOptionsPage();
    }
  });

  chrome.contextMenus.removeAll(() => {
    chrome.contextMenus.create({
      id: 'GPT_Summarizer',
      type: 'normal',
      title: chrome.i18n.getMessage('appAction'),
      contexts: ['page'],
    });
  });

  chrome.contextMenus.onClicked.addListener(async () => {
    const [tab] = await chrome.tabs.query({ active: true });
    if (tab.id) {
      chrome.scripting.executeScript({
        target: { tabId: tab.id },
        files: ['js/popup.js'],
      });
    }
  });

  chrome.action.onClicked.addListener(({ id: tabId }) => {
    chrome.scripting.executeScript({
      target: { tabId },
      files: ['js/popup.js'],
    });
  });

  chrome.runtime.onMessage.addListener((message, _, sendResponse) => {
    (async () => {
      if (message === 'OPEN_OPTIONS_PAGE') {
        chrome.runtime.openOptionsPage();
      } else if (message === 'GET_SETTINGS') {
        const settings = await getSettings();
        sendResponse(settings);
      }
    })();
    return true;
  });
})();
